

//#include <RTL.h>
#include <stdio.h>
#include "cmsis_os2.h"
#include "GUI.h"
#include "DTILite_config.h"
#if (GUI_DEBUG_LEVEL >= 3)
#include "linkmonitor.h"
#endif

/*********************************************************************
*
*       Migrate from RTX to CMSIS-RTOS API2
*/
// Task Management
#define OS_TID              osThreadId_t
#define __task              __NO_RETURN
#define os_tsk_attr(s,v)    (osThreadAttr_t){0,0,0,0,0,s,(osPriority_t)v,0,0}
#define os_dly_wait(t)      osDelay(t)
#define os_tsk_create(p,v)  osThreadNew((osTimerFunc_t)p,NULL,&os_tsk_attr(512,v))
#define os_tsk_self         osThreadGetId

// Semaphore Management
#define OS_SEM              osSemaphoreId_t
#define os_sem_init(p,v)    p=osSemaphoreNew(v,1,NULL)
#define os_sem_send(p)      osSemaphoreRelease(p)
#define os_sem_wait(p,t)    osSemaphoreAcquire(p,t)

// Event Flag Management
#define os_evt_wait_or(f,t) osThreadFlagsWait(f,osFlagsWaitAny,t)
#define os_evt_set(f,p)     osThreadFlagsSet(p,f)

// Time Management
#define os_itv_set(...)
#define os_itv_wait(...)

/*********************************************************************
*
*       Global data
*/
volatile GUI_TIMER_TIME OS_TimeMS;

#if (GUI_X_BACKGROUND_TASK)
/*! @brief background task attributes */
static OS_TID _GuiTaskId;
static const osThreadAttr_t _GuiTaskAttr = {
  .stack_size = configUI_BKG_TASK_STACK_SIZE,   // must be multiple of 8 Bytes
  .priority   = configUI_BG_TASK_PRIORITY,      // set priority to low
};
#endif

#if (defined __CC_ARM)
#pragma push  /* Save existing optimization level */
#pragma O3    /* Optimization level now O3 */
#endif

/*********************************************************************
*
*       Private code
*
**********************************************************************
*/
#if (GUI_X_BACKGROUND_TASK)
/*******************************************************************
*
*       _GUI_Task
*
* This task does the background processing.
* The MainTask job is to update invalid windows, but other things such as
* evaluating mouse or touch input may also be done.
*/
__NO_RETURN static void _GUI_Task(void *pvParameters) {
  (void)pvParameters;
  //vPortTaskUsesFPU();

  while (1) {
    /* Do the background work */
    GUI_Exec();
    GUI_X_ExecIdle();
  }
}
#endif

/*********************************************************************
*
*       Public code
*
**********************************************************************
*/
/*********************************************************************
*
*      Timing:
*                 GUI_X_GetTime()
*                 GUI_X_Delay(int)

  Some timing dependent routines require a GetTime
  and delay function. Default time unit (tick), normally is
  1 ms.
*/

GUI_TIMER_TIME GUI_X_GetTime(void) {
  return OS_TimeMS;
}

void GUI_X_Delay(int ms) {
  while (ms > 0xFFFE) {
    ms -= 0xFFFE;
    os_dly_wait(0xFFFE);
  }
  os_dly_wait(ms);
}

/*********************************************************************
*
*       GUI_X_Init()
*
* Note:
*     GUI_X_Init() is called from GUI_Init is a possibility to init
*     some hardware which needs to be up and running before the GUI.
*     If not required, leave this routine blank.
*/

__task void SysTick_Task(void);
static OS_TID _SysTickTaskId;

void GUI_X_Init(void) {
  _SysTickTaskId = os_tsk_create(SysTick_Task, 24);
#if (GUI_X_BACKGROUND_TASK)
  _GuiTaskId = osThreadNew(_GUI_Task, NULL, &_GuiTaskAttr);
  if (_GuiTaskId == NULL) for (;;) ;
#endif
}

/*********************************************************************
*
*       GUI_X_ExecIdle
*
* Note:
*  Called if WM is in idle state
*/

void GUI_X_ExecIdle(void) {
  os_dly_wait(GUI_X_IDLE_TIME);
}

/*********************************************************************
*
*      Logging: OS dependent

Note:
  Logging is used in higher debug levels only. The typical target
  build does not use logging and does therefor not require any of
  the logging routines below. For a release build without logging
  the routines below may be eliminated to save some space.
  (If the linker is not function aware and eliminates unreferenced
  functions automatically)

*/

#if (GUI_DEBUG_LEVEL < 5)
void GUI_X_Log     (const char *s) { GUI_USE_PARA(s); }
#else
void GUI_X_Log     (const char *s) { DEBUG_MSG("[V] %s\r\n", s); }
#endif
#if (GUI_DEBUG_LEVEL < 4)
void GUI_X_Warn    (const char *s) { GUI_USE_PARA(s); }
#else
void GUI_X_Warn    (const char *s) { DEBUG_MSG("[W] %s\r\n", s); }
#endif
#if (GUI_DEBUG_LEVEL < 3)
void GUI_X_ErrorOut(const char *s) { GUI_USE_PARA(s); }
#else
void GUI_X_ErrorOut(const char *s) { DEBUG_MSG("[E] %s\r\n", s); }
#endif

/*********************************************************************
*
*      Multitasking:
*
*                 GUI_X_InitOS()
*                 GUI_X_GetTaskId()
*                 GUI_X_Lock()
*                 GUI_X_Unlock()
*
* Note:
*   The following routines are required only if emWin is used in a
*   true multi task environment, which means you have more than one
*   thread using the emWin API.
*   In this case the
*                       #define GUI_OS 1
*  needs to be in GUIConf.h
*/

OS_SEM GUI_Semaphore;

void GUI_X_InitOS(void)    { os_sem_init(GUI_Semaphore, 1); }
void GUI_X_Unlock(void)    { os_sem_send(GUI_Semaphore); }
void GUI_X_Lock(void)      { os_sem_wait(GUI_Semaphore, 0xFFFF); }
U32  GUI_X_GetTaskId(void) { return (U32)os_tsk_self(); }

/*********************************************************************
*
*      Event driving (optional with multitasking)
*
*                 GUI_X_WaitEvent()
*                 GUI_X_WaitEventTimed()
*                 GUI_X_SignalEvent()
*/

OS_TID GUI_Task;

void GUI_X_WaitEvent(void) {
  GUI_Task = os_tsk_self();
  os_evt_wait_or(0x0001, 0xFFFF);
}

void GUI_X_WaitEventTimed(int Period) {
  GUI_Task = os_tsk_self();
  os_evt_wait_or(0x0001, (Period > 0xFFFE) ? 0xFFFE : Period);
}

void GUI_X_SignalEvent(void) {
  if (GUI_Task) {
    os_evt_set(0x0001, GUI_Task);
  }
}

/*********************************************************************
*
*       SysTick_Task
*
*/

__task void SysTick_Task(void) {

  os_itv_set(10);
  for (;;) {
    os_itv_wait();
    osDelay(10);
    OS_TimeMS += 10;
  }
}

/*********************************************************************
*
*      GUI_X_Deinit()
*
* Note:
*   Called if a reset for GUI is needed.
*/
void GUI_X_Deinit(void) {
#if (GUI_X_BACKGROUND_TASK)
  if (_GuiTaskId) {
    osThreadTerminate(_GuiTaskId);
    _GuiTaskId = NULL;
  }
#endif
  if (_SysTickTaskId) {
    osThreadTerminate(_SysTickTaskId);
    _SysTickTaskId = NULL;
  }
  if (GUI_Semaphore) {
    osSemaphoreDelete(GUI_Semaphore);
    GUI_Semaphore = NULL;
  }
}

#if (defined __CC_ARM)
#pragma pop   /* Restore original optimization level */
#endif

/*************************** End of file ****************************/
